package com.example.uart2;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.LinkedList;
import java.util.List;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.example.uart2.bean.Uart;
import com.example.uartdemo.SerialPort;
import com.example.uartdemo.Tools;

public class MainActivity extends Activity {

	/** UI **/
	private Button buttonOpen;
	private ListView lv_info;
	private MyAdapter adapter;
	private List<Uart> records=new LinkedList<>();
	
	private MonitorAnalysis_SSTLR1 analysis = new MonitorAnalysis_SSTLR1();

	/** serialport **/
	private SerialPort mSerialPort;
	private InputStream is;
	private OutputStream os;

	private int port = 13;
	private int buad = 9600;

	/** recv Thread **/
	private RecvThread recvThread;

	private boolean isOpen = false;
	private Context context;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		context = this;
		buttonOpen = (Button) findViewById(R.id.buttonOpen);
		lv_info = (ListView) findViewById(R.id.lv_info);
	}

	@Override
	protected void onDestroy() {
		this.close();
		super.onDestroy();
	}

	// open serialport
	private void open() {
		try {
			mSerialPort = new SerialPort(port, buad, 0);
		} catch (Exception e) {
			//Toast.makeText(this, "OpenComport Failed", Toast.LENGTH_LONG).show();
			return;
		}
		is = mSerialPort.getInputStream();
		os = mSerialPort.getOutputStream();
		mSerialPort.power3v3on();
		recvThread = new RecvThread();
		recvThread.start();
		isOpen = true;
		buttonOpen.setText("Stop");
		//Toast.makeText(this, "OpenComport Successfully", Toast.LENGTH_LONG).show();

	}

	// close serialport
	private void close() {
		if (recvThread != null) {
			recvThread.interrupt();
		}
		if (mSerialPort != null) {
			mSerialPort.power3v3off();
			try {
				is.close();
				os.close();
			} catch (IOException e) {
			}
			mSerialPort.close(port);
			isOpen = false;
			buttonOpen.setText("Start");
		}
	}

	public void button_click(View v) {
		int id = v.getId();
		switch (id) {
		case R.id.buttonOpen:
			if (!isOpen)  this.open();
			else  this.close();
			break;
		case R.id.buttonClear:
			if(records != null && records.size() > 0){
				records.clear();
				notifyAdapter();
			}
			break;
		}
	}

	/**
	 * recv thread receive serialport data
	 */
	private class RecvThread extends Thread {
		@Override
		public void run() {
			super.run();
			try {
				while (!isInterrupted()) {
					int size = 0;
					byte[] buffer = new byte[1024];
					if (is == null) return;
					size = is.read(buffer);
					if (size > 0) {
						onDataReceived(buffer, size);
					}
					Thread.sleep(10);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * add recv data on UI
	 */
	private void onDataReceived(final byte[] buffer, final int size) {
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Log.w("data", Tools.Bytes2HexString(buffer, size));
				Uart u=analysis.convertArray(buffer,size);
				if(u!=null){
					for(Uart r:records){
						if(r.getNo().equals(u.getNo())){
							r.setTemp(u.getTemp());
							r.setHumi(u.getHumi());
							r.setTime(u.getTime());
							r.setType(u.getType());
							r.setVoltage(u.getVoltage());
							notifyAdapter();
							Log.w("data", ""+u.toString());
							return;
						}
					}
					records.add(0, u);
					notifyAdapter();
					Log.w("data", ""+u.toString());
				}else{
					Log.w("data", "null");
				}
			}
		});
	}
	
	//֪ͨ
	private void notifyAdapter() { 
		if(adapter == null){
			adapter = new MyAdapter();
			lv_info.setAdapter(adapter);
		}else{
			adapter.notifyDataSetChanged();
		}
		lv_info.setSelection(0);
	}


	private class MyAdapter extends BaseAdapter{

		@Override
		public int getCount() {
			return records.size();
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			Uart u = records.get(position);
			View view;
			ViewHolder holder;
			if(convertView != null && convertView instanceof LinearLayout){
				view = convertView;
				holder = (ViewHolder) view.getTag();
			}else{
				view = View.inflate(context, R.layout.list_item, null);
				holder = new ViewHolder();
				holder.tv_no = (TextView) view.findViewById(R.id.tv_no);
				holder.tv_voltage = (TextView) view.findViewById(R.id.tv_voltage);
				holder.tv_time = (TextView) view.findViewById(R.id.tv_time);
				holder.tv_type = (TextView) view.findViewById(R.id.tv_type);
				view.setTag(holder);
			}
			holder.tv_no.setText(u.getNo() + "");
			holder.tv_voltage.setText(u.getVoltage());
			holder.tv_time.setText(u.getTime());
			holder.tv_time.setTextSize(12);
			holder.tv_type.setText(u.getType());
			return view;
		}
		
		@Override
		public Object getItem(int arg0) {
			return records.get(arg0);
		}

		@Override
		public long getItemId(int arg0) {
			return arg0;
		}
		
	}
	
	static class ViewHolder{
		TextView tv_no;
		TextView tv_voltage;
		TextView tv_time;
		TextView tv_type;
	}
}
